@@ -23,6 +23,7 @@
 #include <QDateTime>
 #include <QDebug>
 #include <QFile>
+#include <QDataStream>
 #include <QMimeDatabase>
 #include <QMimeType>
 #include <QRegularExpression>
@@ -54,7 +55,7 @@
 #include <errno.h>
 #endif
 
-#include <KMessageBox>
+#include <kio_version.h>
 #include <KLocalizedString>
 #include <KRemoteEncoding>
 
@@ -665,8 +666,13 @@ int fishProtocol::establishConnection(const QByteArray &buffer) {
 #else
             return 0;
         } else if (buf.endsWith('?')) {
+#if KIO_VERSION >= QT_VERSION_CHECK(5, 100, 0)
+            int rc = messageBox(QuestionTwoActions, thisFn+buf, QString(), i18nc("@action:button", "Yes"), i18nc("@action:button", "No"));
+            if (rc == KIO::SlaveBase::PrimaryAction) {
+#else
             int rc = messageBox(QuestionYesNo,thisFn+buf);
-            if (rc == KMessageBox::Yes) {
+            if (rc == KIO::SlaveBase::Yes) {
+#endif
                 writeChild("yes\n",4);
             } else {
                 writeChild("no\n",3);
@@ -680,8 +686,13 @@ int fishProtocol::establishConnection(const QByteArray &buffer) {
         }
 #ifdef Q_OS_WIN
         if (buf.endsWith(QLatin1String("(y/n)"))) {
+#if KIO_VERSION >= QT_VERSION_CHECK(5, 100, 0)
+            int rc = messageBox(QuestionTwoActions, thisFn+buf, QString(), i18nc("@action:button", "Yes"), i18nc("@action:button", "No"));
+            if (rc == KIO::SlaveBase::PrimaryAction) {
+#else
             int rc = messageBox(QuestionYesNo,thisFn+buf);
-            if (rc == KMessageBox::Yes) {
+            if (rc == KIO::SlaveBase::Yes) {
+#endif
                 writeChild("y\n",2);
             } else {
                 writeChild("n\n",2);
