@@ -2,6 +2,7 @@
  *  Main implementation for KIO-MTP
  *  SPDX-FileCopyrightText: 2012 Philipp Schmidt <philschmidt@gmx.net>
  *  SPDX-FileCopyrightText: 2018 Andreas Krutzler <andreas.krutzler@gmx.net>
+ *  SPDX-FileCopyrightText: 2022 Harald Sitter <sitter@kde.org>
  *
  *  SPDX-License-Identifier: GPL-2.0-or-later
  */
@@ -16,7 +17,6 @@
 #include <QCoreApplication>
 #include <QTimer>
 
-#include <sys/stat.h>
 #include <sys/types.h>
 #include <utime.h>
 
@@ -27,7 +27,7 @@
 class KIOPluginForMetaData : public QObject
 {
     Q_OBJECT
-    Q_PLUGIN_METADATA(IID "org.kde.kio.slave.mtp" FILE "mtp.json")
+    Q_PLUGIN_METADATA(IID "org.kde.kio.worker.mtp" FILE "mtp.json")
 };
 
 static UDSEntry getEntry(const KMTPDeviceInterface *device)
@@ -88,13 +88,13 @@ static QString urlFileName(const QUrl &url)
     return url.fileName();
 }
 
-static QString convertPath(const QString &slavePath)
+static QString convertPath(const QString &workerPath)
 {
-    return slavePath.section(QLatin1Char('/'), 3, -1, QString::SectionIncludeLeadingSep);
+    return workerPath.section(QLatin1Char('/'), 3, -1, QString::SectionIncludeLeadingSep);
 }
 
 //////////////////////////////////////////////////////////////////////////////
-///////////////////////////// Slave Implementation ///////////////////////////
+///////////////////////////// Worker Implementation ///////////////////////////
 //////////////////////////////////////////////////////////////////////////////
 
 extern "C"
@@ -108,66 +108,70 @@ int Q_DECL_EXPORT kdemain(int argc, char **argv)
         exit(-1);
     }
 
-    MTPSlave slave(argv[2], argv[3]);
+    MTPWorker worker(argv[2], argv[3]);
 
-    slave.dispatchLoop();
+    worker.dispatchLoop();
 
-    qCDebug(LOG_KIO_MTP) << "Slave EventLoop ended";
+    qCDebug(LOG_KIO_MTP) << "Worker EventLoop ended";
 
     return 0;
 }
 
-MTPSlave::MTPSlave(const QByteArray &pool, const QByteArray &app)
-    : SlaveBase("mtp", pool, app)
+MTPWorker::MTPWorker(const QByteArray &pool, const QByteArray &app)
+    : WorkerBase("mtp", pool, app)
 {
-    qCDebug(LOG_KIO_MTP) << "Slave started";
+    qCDebug(LOG_KIO_MTP) << "Worker started";
     qCDebug(LOG_KIO_MTP) << "Connected to kiod5 module:" << m_kmtpDaemon.isValid();
 }
 
-MTPSlave::~MTPSlave()
+MTPWorker::~MTPWorker()
 {
-    qCDebug(LOG_KIO_MTP) << "Slave destroyed";
+    qCDebug(LOG_KIO_MTP) << "Worker destroyed";
 }
 
-int MTPSlave::checkUrl(const QUrl &url, bool redirect)
+enum MTPWorker::Url MTPWorker::checkUrl(const QUrl &url)
 {
-    if (url.path().startsWith(QLatin1String("udi=")) && redirect) {
+    if (url.path().startsWith(QLatin1String("udi="))) {
         const QString udi = url.adjusted(QUrl::StripTrailingSlash).path().remove(0, 4);
 
         qCDebug(LOG_KIO_MTP) << "udi = " << udi;
 
         const KMTPDeviceInterface *device = m_kmtpDaemon.deviceFromUdi(udi);
-        if (device) {
-            QUrl newUrl;
-            newUrl.setScheme(QStringLiteral("mtp"));
-            newUrl.setPath(QLatin1Char('/') + device->friendlyName());
-            redirection(newUrl);
-
-            return 1;
-        } else {
-            return 2;
+        if (!device) {
+            return Url::NotFound;
         }
-    } else if (url.path().startsWith(QLatin1Char('/'))) {
-        return 0;
+
+        QUrl newUrl;
+        newUrl.setScheme(QStringLiteral("mtp"));
+        newUrl.setPath(QLatin1Char('/') + device->friendlyName());
+        redirection(newUrl);
+
+        return Url::Redirected;
     }
-    return -1;
+    if (url.path().startsWith(QLatin1Char('/'))) {
+        return Url::Valid;
+    }
+    if (url.scheme() == QLatin1String("mtp") && url.path().isEmpty()) {
+        QUrl newUrl = url;
+        newUrl.setPath(QLatin1String("/"));
+        redirection(newUrl);
+
+        return Url::Redirected;
+    }
+    return Url::Invalid;
 }
 
-void MTPSlave::listDir(const QUrl &url)
+WorkerResult MTPWorker::listDir(const QUrl &url)
 {
-    const int check = checkUrl(url);
-    switch (check) {
-    case 0:
+    switch (checkUrl(url)) {
+    case Url::Valid:
         break;
-    case 1:
-        finished();
-        return;
-    case 2:
-        error(ERR_DOES_NOT_EXIST, url.path());
-        return;
-    default:
-        error(ERR_MALFORMED_URL, url.path());
-        return;
+    case Url::Redirected:
+        return WorkerResult::pass();
+    case Url::NotFound:
+        return WorkerResult::fail(ERR_DOES_NOT_EXIST, url.path());
+    case Url::Invalid:
+        return WorkerResult::fail(ERR_MALFORMED_URL, url.path());
     }
 
     // list '.' entry, otherwise files cannot be pasted to empty folders
@@ -184,20 +188,19 @@ void MTPSlave::listDir(const QUrl &url)
     if (pathItems.isEmpty()) {
         qCDebug(LOG_KIO_MTP) << "Root directory, listing devices";
 
-        totalSize(filesize_t(m_kmtpDaemon.devices().size()));
-
         const auto devices = m_kmtpDaemon.devices();
+        totalSize(filesize_t(devices.size()));
+
         for (const KMTPDeviceInterface *device : devices) {
             listEntry(getEntry(device));
         }
 
-        qCDebug(LOG_KIO_MTP) << "[SUCCESS] :: Devices:" << m_kmtpDaemon.devices().size();
-        finished();
-        return;
+        qCDebug(LOG_KIO_MTP) << "[SUCCESS] :: Devices:" << devices.size();
+        return WorkerResult::pass();
     }
 
     // traverse into device
-    const KMTPDeviceInterface *mtpDevice = m_kmtpDaemon.deviceFromName(pathItems.first());
+    KMTPDeviceInterface *mtpDevice = m_kmtpDaemon.deviceFromName(pathItems.first());
     if (mtpDevice) {
         // list storage media
         if (pathItems.size() == 1) {
@@ -211,65 +214,53 @@ void MTPSlave::listDir(const QUrl &url)
                     listEntry(getEntry(storage));
                 }
 
-                finished();
                 qCDebug(LOG_KIO_MTP) << "[SUCCESS] :: Storage media:" << storages.count();
-            } else {
-                error(ERR_SLAVE_DEFINED, i18n("No storage media found. Make sure your device is unlocked and has MTP enabled in its USB connection settings."));
+                return WorkerResult::pass();
             }
-        } else {
-            // list files and folders
-            const KMTPStorageInterface *storage = mtpDevice->storageFromDescription(pathItems.at(1));
-            if (storage) {
-                int result;
-                const QString path = convertPath(url.path());
-                const KMTPFileList files = storage->getFilesAndFolders(path, result);
-
-                switch (result) {
-                case 0:
-                    break;
-                case 2:
-                    error(ERR_IS_FILE, url.path());
-                    return;
-                default:
-                    // path not found
-                    error(ERR_CANNOT_ENTER_DIRECTORY, url.path());
-                    return;
-                }
+            return WorkerResult::fail(ERR_WORKER_DEFINED, i18nc("Message shown when attempting to access an MTP device that is not fully accessible yet", "Could not access device. Make sure it is unlocked, and tap \"Allow\" on the popup on its screen. If that does not work, make sure MTP is enabled in its USB connection settings."));
+        }
+        // list files and folders
+        const KMTPStorageInterface *storage = mtpDevice->storageFromDescription(pathItems.at(1));
+        if (storage) {
+            int result;
+            const QString path = convertPath(url.path());
+            const KMTPFileList files = storage->getFilesAndFolders(path, result);
 
+            switch (result) {
+            case 0:
                 for (const KMTPFile &file : files) {
                     listEntry(getEntry(file));
                 }
 
-                finished();
                 qCDebug(LOG_KIO_MTP) << "[SUCCESS] :: Files:" << files.count();
-            } else {
-                // storage not found
-                error(ERR_CANNOT_ENTER_DIRECTORY, url.path());
-                qCDebug(LOG_KIO_MTP) << "[ERROR] :: Storage";
+                return WorkerResult::pass();
+            case 2:
+                return WorkerResult::fail(ERR_IS_FILE, url.path());
             }
+
+            // path not found
+            return WorkerResult::fail(ERR_CANNOT_ENTER_DIRECTORY, url.path());
         }
-    } else {
-        // device not found
-        error(ERR_CANNOT_ENTER_DIRECTORY, url.path());
-        qCDebug(LOG_KIO_MTP) << "[ERROR] :: Device";
+        // storage not found
+        qCDebug(LOG_KIO_MTP) << "[ERROR] :: Storage";
+        return WorkerResult::fail(ERR_CANNOT_ENTER_DIRECTORY, url.path());
     }
+    // device not found
+    qCDebug(LOG_KIO_MTP) << "[ERROR] :: Device";
+    return WorkerResult::fail(ERR_CANNOT_ENTER_DIRECTORY, url.path());
 }
 
-void MTPSlave::stat(const QUrl &url)
+WorkerResult MTPWorker::stat(const QUrl &url)
 {
-    const int check = checkUrl(url);
-    switch (check) {
-    case 0:
+    switch (checkUrl(url)) {
+    case Url::Valid:
         break;
-    case 1:
-        finished();
-        return;
-    case 2:
-        error(ERR_DOES_NOT_EXIST, url.path());
-        return;
-    default:
-        error(ERR_MALFORMED_URL, url.path());
-        return;
+    case Url::Redirected:
+        return WorkerResult::pass();
+    case Url::NotFound:
+        return WorkerResult::fail(ERR_DOES_NOT_EXIST, url.path());
+    case Url::Invalid:
+        return WorkerResult::fail(ERR_MALFORMED_URL, url.path());
     }
 
     const QStringList pathItems = url.path().split(QLatin1Char('/'), Qt::SkipEmptyParts);
@@ -302,40 +293,33 @@ void MTPSlave::stat(const QUrl &url)
                         if (file.isValid()) {
                             entry = getEntry(file);
                         } else {
-                            error(ERR_DOES_NOT_EXIST, url.path());
-                            return;
+                            return WorkerResult::fail(ERR_DOES_NOT_EXIST, url.path());
                         }
                     }
                 } else {
-                    error(ERR_DOES_NOT_EXIST, url.path());
-                    return;
+                    return WorkerResult::fail(ERR_DOES_NOT_EXIST, url.path());
                 }
             }
         } else {
-            error(ERR_DOES_NOT_EXIST, url.path());
-            return;
+            return WorkerResult::fail(ERR_DOES_NOT_EXIST, url.path());
         }
     }
 
     statEntry(entry);
-    finished();
+    return WorkerResult::pass();
 }
 
-void MTPSlave::mimetype(const QUrl &url)
+WorkerResult MTPWorker::mimetype(const QUrl &url)
 {
-    const int check = checkUrl(url);
-    switch (check) {
-    case 0:
+    switch (checkUrl(url)) {
+    case Url::Valid:
         break;
-    case 1:
-        finished();
-        return;
-    case 2:
-        error(ERR_DOES_NOT_EXIST, url.path());
-        return;
-    default:
-        error(ERR_MALFORMED_URL, url.path());
-        return;
+    case Url::Redirected:
+        return WorkerResult::pass();
+    case Url::NotFound:
+        return WorkerResult::fail(ERR_DOES_NOT_EXIST, url.path());
+    case Url::Invalid:
+        return WorkerResult::fail(ERR_MALFORMED_URL, url.path());
     }
 
     const QStringList pathItems = url.path().split(QLatin1Char('/'), Qt::SkipEmptyParts);
@@ -348,27 +332,27 @@ void MTPSlave::mimetype(const QUrl &url)
                 if (file.isValid()) {
                     // NOTE the difference between calling mimetype and mimeType
                     mimeType(file.filetype());
-                    return;
+                    return WorkerResult::pass();
                 }
             }
         }
     } else {
         mimeType(QStringLiteral("inode/directory"));
-        return;
+        return WorkerResult::pass();
     }
 
-    error(ERR_DOES_NOT_EXIST, url.path());
+    return WorkerResult::fail(ERR_DOES_NOT_EXIST, url.path());
 }
 
-void MTPSlave::get(const QUrl &url)
+WorkerResult MTPWorker::get(const QUrl &url)
 {
-    const int check = checkUrl(url);
-    switch (check) {
-    case 0:
+    switch (checkUrl(url)) {
+    case Url::Valid:
         break;
-    default:
-        error(ERR_MALFORMED_URL, url.path());
-        return;
+    case Url::Redirected:
+    case Url::NotFound:
+    case Url::Invalid:
+        return WorkerResult::fail(ERR_MALFORMED_URL, url.path());
     }
 
     const QStringList pathItems = url.path().split(QLatin1Char('/'), Qt::SkipEmptyParts);
@@ -383,8 +367,7 @@ void MTPSlave::get(const QUrl &url)
                 const QString path = convertPath(url.path());
                 const KMTPFile source = storage->getFileMetadata(path);
                 if (!source.isValid()) {
-                    error(KIO::ERR_DOES_NOT_EXIST, url.path());
-                    return;
+                    return WorkerResult::fail(KIO::ERR_DOES_NOT_EXIST, url.path());
                 }
 
                 mimeType(source.filetype());
@@ -392,13 +375,12 @@ void MTPSlave::get(const QUrl &url)
 
                 int result = storage->getFileToHandler(path);
                 if (result) {
-                    error(KIO::ERR_CANNOT_READ, url.path());
-                    return;
+                    return WorkerResult::fail(KIO::ERR_CANNOT_READ, url.path());
                 }
 
                 QEventLoop loop;
                 connect(storage, &KMTPStorageInterface::dataReady, &loop, [this] (const QByteArray &data) {
-                    MTPSlave::data(data);
+                    MTPWorker::data(data);
                 });
                 connect(storage, &KMTPStorageInterface::copyFinished, &loop, &QEventLoop::exit);
                 result = loop.exec();
@@ -406,39 +388,35 @@ void MTPSlave::get(const QUrl &url)
                 qCDebug(LOG_KIO_MTP) << "data received";
 
                 if (result) {
-                    error(ERR_CANNOT_READ, url.path());
-                    return;
+                    return WorkerResult::fail(ERR_CANNOT_READ, url.path());
                 }
 
                 data(QByteArray());
-                finished();
-                return;
+                return WorkerResult::pass();
             }
         }
     } else {
-        error(ERR_UNSUPPORTED_ACTION, url.path());
-        return;
+        return WorkerResult::fail(ERR_UNSUPPORTED_ACTION, url.path());
     }
-    error(ERR_CANNOT_READ, url.path());
+    return WorkerResult::fail(ERR_CANNOT_READ, url.path());
 }
 
-void MTPSlave::put(const QUrl &url, int, JobFlags flags)
+WorkerResult MTPWorker::put(const QUrl &url, int, JobFlags flags)
 {
-    const int check = checkUrl(url);
-    switch (check) {
-    case 0:
+    switch (checkUrl(url)) {
+    case Url::Valid:
         break;
-    default:
-        error(ERR_MALFORMED_URL, url.path());
-        return;
+    case Url::Redirected:
+    case Url::NotFound:
+    case Url::Invalid:
+        return WorkerResult::fail(ERR_MALFORMED_URL, url.path());
     }
 
     const QStringList destItems = url.path().split(QLatin1Char('/'), Qt::SkipEmptyParts);
 
     // can't copy to root or device, needs storage
     if (destItems.size() < 2) {
-        error(ERR_UNSUPPORTED_ACTION, url.path());
-        return;
+        return WorkerResult::fail(ERR_UNSUPPORTED_ACTION, url.path());
     }
 
     // we need to get the entire file first, then we can upload
@@ -468,12 +446,10 @@ void MTPSlave::put(const QUrl &url, int, JobFlags flags)
                         // delete existing file on the device
                         const int result = storage->deleteObject(destinationPath);
                         if (result) {
-                            error(ERR_CANNOT_DELETE, url.path());
-                            return;
+                            return WorkerResult::fail(ERR_CANNOT_DELETE, url.path());
                         }
                     } else {
-                        error(ERR_FILE_ALREADY_EXIST, url.path());
-                        return;
+                        return WorkerResult::fail(ERR_FILE_ALREADY_EXIST, url.path());
                     }
                 }
 
@@ -482,8 +458,7 @@ void MTPSlave::put(const QUrl &url, int, JobFlags flags)
                 QDBusUnixFileDescriptor descriptor(temp.handle());
                 int result = storage->sendFileFromFileDescriptor(descriptor, destinationPath);
                 if (result) {
-                    error(KIO::ERR_CANNOT_WRITE, urlFileName(url));
-                    return;
+                    return WorkerResult::fail(KIO::ERR_CANNOT_WRITE, urlFileName(url));
                 }
 
                 result = waitForCopyOperation(storage);
@@ -492,51 +467,46 @@ void MTPSlave::put(const QUrl &url, int, JobFlags flags)
 
                 switch (result) {
                 case 0:
-                    break;
+                    qCDebug(LOG_KIO_MTP) << "data sent";
+                    return WorkerResult::pass();
                 case 2:
-                    error(ERR_IS_FILE, urlDirectory(url));
-                    return;
-                default:
-                    error(KIO::ERR_CANNOT_WRITE, urlFileName(url));
-                    return;
+                    return WorkerResult::fail(ERR_IS_FILE, urlDirectory(url));
                 }
 
-                qCDebug(LOG_KIO_MTP) << "data sent";
-                finished();
-                return;
+                return WorkerResult::fail(KIO::ERR_CANNOT_WRITE, urlFileName(url));
             }
         }
     }
 
-    error(KIO::ERR_CANNOT_WRITE, urlFileName(url));
+    return WorkerResult::fail(KIO::ERR_CANNOT_WRITE, urlFileName(url));
 }
 
-void MTPSlave::copy(const QUrl &src, const QUrl &dest, int, JobFlags flags)
+WorkerResult MTPWorker::copy(const QUrl &src, const QUrl &dest, int, JobFlags flags)
 {
     if (src.scheme() == QLatin1String("mtp") && dest.scheme() == QLatin1String("mtp")) {
         qCDebug(LOG_KIO_MTP) << "Copy on device: Not supported";
         // MTP doesn't support moving files directly on the device, so we have to download and then upload...
 
-        error(ERR_UNSUPPORTED_ACTION, i18n("Cannot copy/move files on the device itself"));
-        return;
-    } else if (src.scheme() == QLatin1String("file") && dest.scheme() == QLatin1String("mtp")) {
+        return WorkerResult::fail(ERR_UNSUPPORTED_ACTION, i18n("Cannot copy/move files on the device itself"));
+    }
+    if (src.scheme() == QLatin1String("file") && dest.scheme() == QLatin1String("mtp")) {
         // copy from filesystem to the device
 
-        const int check = checkUrl(dest);
-        switch (check) {
-        case 0:
+        switch (checkUrl(dest)) {
+        case Url::Valid:
             break;
-        default:
-            error(ERR_MALFORMED_URL, dest.path());
-            return;
+        case Url::NotFound:
+            return WorkerResult::fail(ERR_DOES_NOT_EXIST, src.path());
+        case Url::Redirected:
+        case Url::Invalid:
+            return WorkerResult::fail(ERR_MALFORMED_URL, dest.path());
         }
 
         QStringList destItems = dest.path().split(QLatin1Char('/'), Qt::SkipEmptyParts);
 
         // can't copy to root or device, needs storage
         if (destItems.size() < 2) {
-            error(ERR_UNSUPPORTED_ACTION, dest.path());
-            return;
+            return WorkerResult::fail(ERR_UNSUPPORTED_ACTION, dest.path());
         }
 
         qCDebug(LOG_KIO_MTP) << "Copy file " << urlFileName(src) << "from filesystem to device" << urlDirectory(src, true) << urlDirectory(dest, true);
@@ -554,19 +524,16 @@ void MTPSlave::copy(const QUrl &src, const QUrl &dest, int, JobFlags flags)
                         // delete existing file on the device
                         const int result = storage->deleteObject(destinationPath);
                         if (result) {
-                            error(ERR_CANNOT_DELETE, dest.path());
-                            return;
+                            return WorkerResult::fail(ERR_CANNOT_DELETE, dest.path());
                         }
                     } else {
-                        error(ERR_FILE_ALREADY_EXIST, dest.path());
-                        return;
+                        return WorkerResult::fail(ERR_FILE_ALREADY_EXIST, dest.path());
                     }
                 }
 
                 QFile srcFile(src.path());
                 if (!srcFile.open(QIODevice::ReadOnly)) {
-                    error(KIO::ERR_CANNOT_OPEN_FOR_READING, src.path());
-                    return;
+                    return WorkerResult::fail(KIO::ERR_CANNOT_OPEN_FOR_READING, src.path());
                 }
 
                 qCDebug(LOG_KIO_MTP) << "Sending file" << srcFile.fileName() << "with size" << srcFile.size();
@@ -576,8 +543,7 @@ void MTPSlave::copy(const QUrl &src, const QUrl &dest, int, JobFlags flags)
                 QDBusUnixFileDescriptor descriptor(srcFile.handle());
                 int result = storage->sendFileFromFileDescriptor(descriptor, destinationPath);
                 if (result) {
-                    error(KIO::ERR_CANNOT_WRITE, urlFileName(dest));
-                    return;
+                    return WorkerResult::fail(KIO::ERR_CANNOT_WRITE, urlFileName(dest));
                 }
 
                 result = waitForCopyOperation(storage);
@@ -585,27 +551,26 @@ void MTPSlave::copy(const QUrl &src, const QUrl &dest, int, JobFlags flags)
                 srcFile.close();
 
                 if (result) {
-                    error(KIO::ERR_CANNOT_WRITE, urlFileName(dest));
-                    return;
+                    return WorkerResult::fail(KIO::ERR_CANNOT_WRITE, urlFileName(dest));
                 }
 
                 qCDebug(LOG_KIO_MTP) << "Sent file";
-                finished();
-                return;
+                return WorkerResult::pass();
             }
         }
-        error(KIO::ERR_CANNOT_WRITE, urlFileName(src));
-
-    } else if (src.scheme() == QLatin1String("mtp") && dest.scheme() == QLatin1String("file")) {
+        return WorkerResult::fail(KIO::ERR_CANNOT_WRITE, urlFileName(src));
+    }
+    if (src.scheme() == QLatin1String("mtp") && dest.scheme() == QLatin1String("file")) {
         // copy from the device to filesystem
 
-        const int check = checkUrl(src);
-        switch (check) {
-        case 0:
+        switch (checkUrl(src)) {
+        case Url::Valid:
             break;
-        default:
-            error(ERR_MALFORMED_URL, src.path());
-            return;
+        case Url::NotFound:
+            return WorkerResult::fail(ERR_DOES_NOT_EXIST, src.toDisplayString());
+        case Url::Redirected:
+        case Url::Invalid:
+            return WorkerResult::fail(ERR_MALFORMED_URL, src.toDisplayString());
         }
 
         qCDebug(LOG_KIO_MTP) << "Copy file" << urlFileName(src) << "from device to filesystem" << urlDirectory(src, true) << urlDirectory(dest, true);
@@ -613,16 +578,14 @@ void MTPSlave::copy(const QUrl &src, const QUrl &dest, int, JobFlags flags)
         QFileInfo destination(dest.path());
 
         if (!(flags & KIO::Overwrite) && destination.exists()) {
-            error(ERR_FILE_ALREADY_EXIST, dest.path());
-            return;
+            return WorkerResult::fail(ERR_FILE_ALREADY_EXIST, dest.path());
         }
 
         const QStringList srcItems = src.path().split(QLatin1Char('/'), Qt::SkipEmptyParts);
 
         // can't copy to root or device, needs storage
         if (srcItems.size() < 2) {
-            error(ERR_UNSUPPORTED_ACTION, src.path());
-            return;
+            return WorkerResult::fail(ERR_UNSUPPORTED_ACTION, src.path());
         }
 
         const KMTPDeviceInterface *mtpDevice = m_kmtpDaemon.deviceFromName(srcItems.first());
@@ -632,14 +595,12 @@ void MTPSlave::copy(const QUrl &src, const QUrl &dest, int, JobFlags flags)
 
                 QFile destFile(dest.path());
                 if (!destFile.open(QIODevice::Truncate | QIODevice::WriteOnly)) {
-                    error(KIO::ERR_WRITE_ACCESS_DENIED, dest.path());
-                    return;
+                    return WorkerResult::fail(KIO::ERR_WRITE_ACCESS_DENIED, dest.path());
                 }
 
                 const KMTPFile source = storage->getFileMetadata(convertPath(src.path()));
                 if (!source.isValid()) {
-                    error(KIO::ERR_DOES_NOT_EXIST, src.path());
-                    return;
+                    return WorkerResult::fail(KIO::ERR_DOES_NOT_EXIST, src.path());
                 }
 
                 totalSize(source.filesize());
@@ -647,8 +608,7 @@ void MTPSlave::copy(const QUrl &src, const QUrl &dest, int, JobFlags flags)
                 QDBusUnixFileDescriptor descriptor(destFile.handle());
                 int result = storage->getFileToFileDescriptor(descriptor, convertPath(src.path()));
                 if (result) {
-                    error(KIO::ERR_CANNOT_READ, urlFileName(src));
-                    return;
+                    return WorkerResult::fail(KIO::ERR_CANNOT_READ, urlFileName(src));
                 }
 
                 result = waitForCopyOperation(storage);
@@ -656,8 +616,7 @@ void MTPSlave::copy(const QUrl &src, const QUrl &dest, int, JobFlags flags)
                 destFile.close();
 
                 if (result) {
-                    error(KIO::ERR_CANNOT_READ, urlFileName(src));
-                    return;
+                    return WorkerResult::fail(KIO::ERR_CANNOT_READ, urlFileName(src));
                 }
 
                 // set correct modification time
@@ -668,23 +627,23 @@ void MTPSlave::copy(const QUrl &src, const QUrl &dest, int, JobFlags flags)
                 ::utime(dest.path().toUtf8().data(), &times);
 
                 qCDebug(LOG_KIO_MTP) << "Received file";
-                finished();
-                return;
+                return WorkerResult::pass();
             }
         }
-        error(KIO::ERR_CANNOT_READ, urlFileName(src));
+        return WorkerResult::fail(KIO::ERR_CANNOT_READ, urlFileName(src));
     }
+    return WorkerResult::fail(ERR_UNSUPPORTED_ACTION, {});
 }
 
-void MTPSlave::mkdir(const QUrl &url, int)
+WorkerResult MTPWorker::mkdir(const QUrl &url, int)
 {
-    const int check = checkUrl(url);
-    switch (check) {
-    case 0:
+    switch (checkUrl(url)) {
+    case Url::Valid:
         break;
-    default:
-        error(ERR_MALFORMED_URL, url.path());
-        return;
+    case Url::Redirected:
+    case Url::NotFound:
+    case Url::Invalid:
+        return WorkerResult::fail(ERR_MALFORMED_URL, url.path());
     }
 
     const QStringList pathItems = url.path().split(QLatin1Char('/'), Qt::SkipEmptyParts);
@@ -696,24 +655,23 @@ void MTPSlave::mkdir(const QUrl &url, int)
                 // TODO: folder already exists
                 const quint32 itemId = storage->createFolder(convertPath(url.path()));
                 if (itemId) {
-                    finished();
-                    return;
+                    return WorkerResult::pass();
                 }
             }
         }
     }
-    error(ERR_CANNOT_MKDIR, url.path());
+    return WorkerResult::fail(ERR_CANNOT_MKDIR, url.path());
 }
 
-void MTPSlave::del(const QUrl &url, bool)
+WorkerResult MTPWorker::del(const QUrl &url, bool)
 {
-    const int check = checkUrl(url);
-    switch (check) {
-    case 0:
+    switch (checkUrl(url)) {
+    case Url::Valid:
         break;
-    default:
-        error(ERR_MALFORMED_URL, url.path());
-        return;
+    case Url::Redirected:
+    case Url::NotFound:
+    case Url::Invalid:
+        return WorkerResult::fail(ERR_MALFORMED_URL, url.path());
     }
 
     const QStringList pathItems = url.path().split(QLatin1Char('/'), Qt::SkipEmptyParts);
@@ -724,40 +682,39 @@ void MTPSlave::del(const QUrl &url, bool)
             if (storage) {
                 const int result = storage->deleteObject(convertPath(url.path()));
                 if (!result) {
-                    finished();
-                    return;
+                    return WorkerResult::pass();
                 }
             }
         }
     }
-    error(ERR_CANNOT_DELETE, url.path());
+    return WorkerResult::fail(ERR_CANNOT_DELETE, url.path());
 }
 
-void MTPSlave::rename(const QUrl &src, const QUrl &dest, JobFlags flags)
+WorkerResult MTPWorker::rename(const QUrl &src, const QUrl &dest, JobFlags flags)
 {
-    int check = checkUrl(src);
-    switch (check) {
-    case 0:
+    switch (checkUrl(src)) {
+    case Url::Valid:
         break;
-    default:
-        error(ERR_MALFORMED_URL, src.path());
-        return;
+    case Url::Redirected:
+    case Url::NotFound:
+    case Url::Invalid:
+        return WorkerResult::fail(ERR_MALFORMED_URL, src.path());
     }
-    check = checkUrl(dest);
-    switch (check) {
-    case 0:
+
+    switch (checkUrl(dest)) {
+    case Url::Valid:
         break;
-    default:
-        error(ERR_MALFORMED_URL, dest.path());
-        return;
+    case Url::Redirected:
+    case Url::NotFound:
+    case Url::Invalid:
+        return WorkerResult::fail(ERR_MALFORMED_URL, dest.path());
     }
 
     if (src.scheme() != QLatin1String("mtp")) {
         // Kate: when editing files directly on the device and the user wants to save the changes,
         // Kate tries to move the file from /tmp/xxx to the MTP device which is not supported.
         // The ERR_UNSUPPORTED_ACTION error tells Kate to copy the file instead of moving.
-        error(ERR_UNSUPPORTED_ACTION, src.path());
-        return;
+        return WorkerResult::fail(ERR_UNSUPPORTED_ACTION, src.path());
     }
 
     const QStringList srcItems = src.path().split(QLatin1Char('/'), Qt::SkipEmptyParts);
@@ -767,14 +724,12 @@ void MTPSlave::rename(const QUrl &src, const QUrl &dest, JobFlags flags)
         if (srcItems.size() == 1) {
             const int result = mtpDevice->setFriendlyName(urlFileName(dest));
             if (!result) {
-                finished();
-                return;
+                return WorkerResult::pass();
             }
         }
         // rename Storage
         else if (srcItems.size() == 2) {
-            error(ERR_CANNOT_RENAME, src.path());
-            return;
+            return WorkerResult::fail(ERR_CANNOT_RENAME, src.path());
         } else {
             // rename file or folder
             const KMTPStorageInterface *storage = mtpDevice->storageFromDescription(srcItems.at(1));
@@ -788,56 +743,36 @@ void MTPSlave::rename(const QUrl &src, const QUrl &dest, JobFlags flags)
                         // delete existing file on the device
                         const int result = storage->deleteObject(destinationPath);
                         if (result) {
-                            error(ERR_CANNOT_DELETE, dest.path());
-                            return;
+                            return WorkerResult::fail(ERR_CANNOT_DELETE, dest.path());
                         }
                     } else {
-                        error(ERR_FILE_ALREADY_EXIST, dest.path());
-                        return;
+                        return WorkerResult::fail(ERR_FILE_ALREADY_EXIST, dest.path());
                     }
                 }
 
                 const int result = storage->setFileName(convertPath(src.path()), dest.fileName());
                 if (!result) {
-                    finished();
-                    return;
+                    return WorkerResult::pass();
                 }
             }
         }
     }
-    error(ERR_CANNOT_RENAME, src.path());
+    return WorkerResult::fail(ERR_CANNOT_RENAME, src.path());
 }
 
-void MTPSlave::virtual_hook(int id, void *data)
-{
-    switch(id) {
-    case SlaveBase::GetFileSystemFreeSpace: {
-        QUrl *url = static_cast<QUrl *>(data);
-        fileSystemFreeSpace(*url);
-    }
-    break;
-    default:
-        SlaveBase::virtual_hook(id, data);
-    }
-}
-
-void MTPSlave::fileSystemFreeSpace(const QUrl &url)
+WorkerResult MTPWorker::fileSystemFreeSpace(const QUrl &url)
 {
     qCDebug(LOG_KIO_MTP) << "fileSystemFreeSpace:" << url;
 
-    const int check = checkUrl(url);
-    switch (check) {
-    case 0:
+    switch (checkUrl(url)) {
+    case Url::Valid:
         break;
-    case 1:
-        finished();
-        return;
-    case 2:
-        error(ERR_DOES_NOT_EXIST, url.toDisplayString());
-        return;
-    default:
-        error(ERR_MALFORMED_URL, url.toDisplayString());
-        return;
+    case Url::Redirected:
+        return WorkerResult::pass();
+    case Url::NotFound:
+        return WorkerResult::fail(ERR_DOES_NOT_EXIST, url.path());
+    case Url::Invalid:
+        return WorkerResult::fail(ERR_MALFORMED_URL, url.path());
     }
 
     const QStringList pathItems = url.path().split(QLatin1Char('/'), Qt::SkipEmptyParts);
@@ -850,15 +785,14 @@ void MTPSlave::fileSystemFreeSpace(const QUrl &url)
             if (storage) {
                 setMetaData(QStringLiteral("total"), QString::number(storage->maxCapacity()));
                 setMetaData(QStringLiteral("available"), QString::number(storage->freeSpaceInBytes()));
-                finished();
-                return;
+                return WorkerResult::pass();
             }
         }
     }
-    error(KIO::ERR_CANNOT_STAT, url.toDisplayString());
+    return WorkerResult::fail(KIO::ERR_CANNOT_STAT, url.toDisplayString());
 }
 
-int MTPSlave::waitForCopyOperation(const KMTPStorageInterface *storage)
+int MTPWorker::waitForCopyOperation(const KMTPStorageInterface *storage)
 {
     QEventLoop loop;
     connect(storage, &KMTPStorageInterface::copyProgress, &loop, [this] (qulonglong sent, qulonglong total) {
@@ -866,7 +800,7 @@ int MTPSlave::waitForCopyOperation(const KMTPStorageInterface *storage)
         processedSize(sent);
     });
 
-    // any chance to 'miss' the copyFinished signal and dead lock the slave?
+    // any chance to 'miss' the copyFinished signal and dead lock the worker?
     connect(storage, &KMTPStorageInterface::copyFinished, &loop, &QEventLoop::exit);
     return loop.exec();
 }
