@@ -178,11 +178,11 @@ static QString getMimetype(LIBMTP_filetype_t filetype)
     case LIBMTP_FILETYPE_OGG:
         return QStringLiteral("audio/x-vorbis+ogg");
     case LIBMTP_FILETYPE_AUDIBLE:
-        return QString();
+        return {};
     case LIBMTP_FILETYPE_MP4:
-        return QStringLiteral("audio/mp4");
+        return QStringLiteral("video/mp4");
     case LIBMTP_FILETYPE_UNDEF_AUDIO:
-        return QString();
+        return {};
     case LIBMTP_FILETYPE_WMV:
         return QStringLiteral("video/x-ms-wmv");
     case LIBMTP_FILETYPE_AVI:
@@ -194,11 +194,11 @@ static QString getMimetype(LIBMTP_filetype_t filetype)
     case LIBMTP_FILETYPE_QT:
         return QStringLiteral("video/quicktime");
     case LIBMTP_FILETYPE_UNDEF_VIDEO:
-        return QString();
+        return {};
     case LIBMTP_FILETYPE_JPEG:
         return QStringLiteral("image/jpeg");
     case LIBMTP_FILETYPE_JFIF:
-        return QString();
+        return {};
     case LIBMTP_FILETYPE_TIFF:
         return QStringLiteral("image/tiff");
     case LIBMTP_FILETYPE_BMP:
@@ -226,11 +226,11 @@ static QString getMimetype(LIBMTP_filetype_t filetype)
     case LIBMTP_FILETYPE_HTML:
         return QStringLiteral("text/html");
     case LIBMTP_FILETYPE_FIRMWARE:
-        return QString();
+        return {};
     case LIBMTP_FILETYPE_AAC:
         return QStringLiteral("audio/aac");
     case LIBMTP_FILETYPE_MEDIACARD:
-        return QString();
+        return {};
     case LIBMTP_FILETYPE_FLAC:
         return QStringLiteral("audio/flac");
     case LIBMTP_FILETYPE_MP2:
@@ -246,11 +246,13 @@ static QString getMimetype(LIBMTP_filetype_t filetype)
     case LIBMTP_FILETYPE_PPT:
         return QStringLiteral("application/vnd.ms-powerpoint");
     case LIBMTP_FILETYPE_MHT:
-        return QString();
+        return {};
     case LIBMTP_FILETYPE_JP2:
         return QStringLiteral("image/jpeg2000");
     case LIBMTP_FILETYPE_JPX:
         return QStringLiteral("application/x-jbuilder-project");
+    case LIBMTP_FILETYPE_ALBUM:
+    case LIBMTP_FILETYPE_PLAYLIST:
     case LIBMTP_FILETYPE_UNKNOWN:
         return {};
     }
@@ -392,7 +394,7 @@ KMTPFile MTPStorage::getFileFromPath(const QString &path)
             currentParent = it->itemId();
         } else {
             qCDebug(LOG_KIOD_KMTPD) << "File not found!";
-            return KMTPFile();
+            return {};
         }
         currentPath.append(QLatin1Char('/') + element);
     }
@@ -404,7 +406,7 @@ KMTPFile MTPStorage::getFileFromPath(const QString &path)
         const KMTPFile mtpFile = createMTPFile(file);
         return mtpFile;
     }
-    return KMTPFile();
+    return {};
 }
 
 KMTPFileList MTPStorage::getFilesAndFoldersCached(const QString &path, quint32 parentId)
@@ -433,10 +435,9 @@ quint32 MTPStorage::queryPath(const QString &path, int timeToLive)
             item.first = dateTime.addSecs(timeToLive);
             m_cache.insert(path, item);
             return item.second;
-        } else {
-            m_cache.remove(path);
-            return 0;
         }
+        m_cache.remove(path);
+        return 0;
     }
 
     return 0;
@@ -467,10 +468,11 @@ KMTPFileList MTPStorage::getFilesAndFolders(const QString &path, int &result)
     const KMTPFile file = getFileFromPath(path);
     if (!file.isValid()) {
         result = 1;     // not existing
-        return KMTPFileList();
-    } else if (!file.isFolder()) {
+        return {};
+    }
+    if (!file.isFolder()) {
         result = 2;     // is file
-        return KMTPFileList();
+        return {};
     }
     return getFilesAndFoldersCached(path, file.itemId());
 }
@@ -506,22 +508,23 @@ int MTPStorage::getFileToFileDescriptor(const QDBusUnixFileDescriptor &descripto
     qCDebug(LOG_KIOD_KMTPD) << "getFileToFileDescriptor:" << sourcePath;
 
     const KMTPFile source = getFileMetadata(sourcePath);
-    if (source.isValid()) {
-        const quint32 itemId = source.itemId();
-
-        // big files take some time to copy, and this may lead into D-Bus timeouts.
-        // therefore the actual copying is not done within the D-Bus method itself but right after we return to the event loop
-        QTimer::singleShot(0, this, [this, itemId, descriptor] {
-            const int result = LIBMTP_Get_File_To_File_Descriptor(getDevice(), itemId, descriptor.fileDescriptor(), onDataProgress, this);
-            if (result) {
-                LIBMTP_Dump_Errorstack(getDevice());
-                LIBMTP_Clear_Errorstack(getDevice());
-            }
-            Q_EMIT copyFinished(result);
-        });
-        return 0;
+    if (!source.isValid()) {
+        return 1;
     }
-    return 1;
+
+    const quint32 itemId = source.itemId();
+
+    // big files take some time to copy, and this may lead into D-Bus timeouts.
+    // therefore the actual copying is not done within the D-Bus method itself but right after we return to the event loop
+    QTimer::singleShot(0, this, [this, itemId, descriptor] {
+        const int result = LIBMTP_Get_File_To_File_Descriptor(getDevice(), itemId, descriptor.fileDescriptor(), onDataProgress, this);
+        if (result) {
+            LIBMTP_Dump_Errorstack(getDevice());
+            LIBMTP_Clear_Errorstack(getDevice());
+        }
+        Q_EMIT copyFinished(result);
+    });
+    return 0;
 }
 
 int MTPStorage::sendFileFromFileDescriptor(const QDBusUnixFileDescriptor &descriptor, const QString &destinationPath)
